/*
 * Copyright (c) 1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, and distribute this software for
 * NON-COMMERCIAL purposes and without fee is hereby granted
 * provided that this copyright notice appears in all copies.
 *
 */

import java.sql.*;
import javax.swing.*;
import java.util.*;

/**
 * ėpf[^x[XEANZXEAvP[VB
 * @author      Tadayoshi Maekawa, Sun Microsystems K.K.
 * @version     1.2 99/09/24
 */
public class DBBrowser {
    /**
     * 萔:e[uE^Cv:[UEe[u
     */
    public static final int TYPE_USER_TABLE = 0;

    /**
     * 萔:e[uE^Cv:VXeEe[u
     */
    public static final int TYPE_SYSTEM_TABLE = 1;

    /**
     * GUI ւ̃t@XB
     */
    DBBrowserUI ui;

    /**
     * f[^x[XڑB
     */
    Connection conn;

    /**
     * f[^x[XڑvpeBB
     */
    Properties dbProps;

    /**
     * \̃e[uێ HashtableB
     */
    Hashtable displayingTables;

    /**
     * main \bhB
     */
    public static void main(String[] a) {
        DBBrowser browser = new DBBrowser();
        browser.start();
    }

    /**
     * RXgN^B
     */
    public DBBrowser() {
        dbProps = new Properties();
        displayingTables = new Hashtable();
        // GUI
        ui = new DBBrowserUI(this);
        ui.setVisible(true);
    }

    /**
     * NB
     */
    protected void start() {
        // DB ڑ̊m
        getDBConnection();
    }

    /**
     * IB
     */
    protected void stop() {
        dropDBConnection();
        if (ui != null) {
            try {
                ui.dispose();
            } catch (Exception ex) {}
            ui = null;
        }
        System.exit(0);
    }

    /**
     * e[uE^Cvݒ肷B
     * @param tp ^Cvl
     */
    protected void setupType(int tp) {
        if (tp != TYPE_SYSTEM_TABLE && tp != TYPE_USER_TABLE) {
            tp = TYPE_USER_TABLE;
        }
        try {
            retrieveTables(tp);
        } catch (CanNotRetrieveTablesException ex) {
            ui.showError(ex.getMessage());
            ex.printStackTrace();
        }
    }

    /**
     * e[u\B
     * @param nm e[u
     * @exception TableNotShowedException e[u̕\ɎsꍇB
     */
    protected void showTable(String nm) {
        TableViewer tv;
        if (displayingTables.get(nm) != null) {
            return;
        }
        synchronized (displayingTables) {
            tv = new TableViewer(this, conn, nm);
            displayingTables.put(nm, tv);
        }
        tv.start();
    }

    /**
     * e[u\ȈsBe[uEGfB^ʒmB
     * @param nm e[u
     */
    protected void hidedTable(String nm) {
        TableViewer te;
        synchronized (displayingTables) {
            te = (TableViewer)displayingTables.remove(nm);
        }
    }

    /**
     * Login _CAOgp DB ڑ擾B
     */
    protected void getDBConnection() {
        Connection cn;
        Login login;
        dropDBConnection();
        ui.setEnabledAll(false);
        login = new Login(ui, true, dbProps);
        cn = login.getConnection();
        if (cn != null) {
            ui.setEnabledAll(cn != null);
        }
        conn = cn;
    }

    /**
     * DB ڑpB
     */
    protected void dropDBConnection() {
        try {
            if (conn != null) {
                conn.close();
            }
            conn = null;
        } catch (SQLException ex) {}
        ui.setEnabledAll(false);
    }

    /**
     * vpeBҏW_CAOgpāADB ڑvpeBݒ肷B
     */
    protected void setDBProperty() {
        Properties pr;
        PropertyEditor peditor;
        pr = (Properties)dbProps.clone();
        peditor = new PropertyEditor(ui, true, pr);
        pr = peditor.getProperties();
        dbProps = pr;
    }

    /**
     * w肳ꂽe[u^Cṽe[uꗗ擾B
     * @param tp ^Cvl
     * @exception CanNotRetrieveTablesException ꗗ擾ɎsꍇB
     */
    private void retrieveTables(int tp)
    throws CanNotRetrieveTablesException {
        String type;
        if (tp == TYPE_SYSTEM_TABLE) {
            type = "SYSTEM_TABLE";
        } else {
            type = "TABLE";
        }
        //
        String[] ret = null;
        try {
            DatabaseMetaData dmd;
            ResultSet rs;
            Vector tbls = new Vector();
            String[] tps = new String[] { type };
            dmd = conn.getMetaData();
            rs = dmd.getTables(null, "%", "%", tps);
            while (rs.next()) {
                tbls.add(rs.getString("TABLE_NAME").trim());
            }
            rs.close();
            if (tbls.size() > 0) {
                ret = new String[tbls.size()];
                tbls.copyInto(ret);
            } else {
                throw new CanNotRetrieveTablesException(
                    "w肳ꂽ^Cṽe[u݂܂B");
            }
        } catch (SQLException ex) {
            throw new CanNotRetrieveTablesException(
                "ꗗ擾Ɏs܂B", ex);
        }
        ui.displayTables(ret);
    }
}
