/*
 * Copyright (c) 1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, and distribute this software for
 * NON-COMMERCIAL purposes and without fee is hereby granted
 * provided that this copyright notice appears in all copies.
 *
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.Vector;

/**
 * DB uEU GUI 񋟂B
 * @author Tadayoshi Maekawa, Sun Microsystems K.K.
 * @version 1.2 99/09/24
 */
class DBBrowserUI extends JFrame implements ActionListener {
    /**
     * AvP[V{́B
     */
    DBBrowser application;

    /**
     * GUI IuWFNgB
     */
    ButtonGroup tableTypeGroup;
    JToggleButton userTableButton;
    JToggleButton systemTableButton;
    JPanel tableDisplayPanel;
    JScrollPane displayScrollPane;

    Vector tableButtons;

    JMenuBar menu;
    JMenu connectMenu;
    JMenuItem connectItem;
    JMenuItem cutOffItem;
    JMenuItem propertyItem;
    JMenuItem exitItem;

    /**
     * action-command
     */
    final String CMD_USER_TABLE = "USER TABLE";
    final String CMD_SYSTEM_TABLE = "SYSTEM TABLE";
    final String CMD_CONNECT = "CONNECT";
    final String CMD_CUTOFF = "CUTOFF";
    final String CMD_PROPERTY = "PROPERTY";
    final String CMD_EXIT = "EXIT";

    /**
     * RXgN^B
     * @param ap AvP[V{
     */
    public DBBrowserUI(DBBrowser ap) {
        super("DB Browser");
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        application = ap;
        tableButtons = new Vector();
        //
        GridBagLayout gbl = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();
        Container pane = getContentPane();
        tableTypeGroup = new ButtonGroup();
        pane.setLayout(gbl);
        gbc.insets = new Insets(3, 5, 3, 5);
        //
        place(gbc, pane, new JLabel("e[uE^Cv"),
              0, 0, 1, 1, 0.0, 0.0, gbc.CENTER, gbc.NONE);
        userTableButton = new JToggleButton("[UEe[u");
        userTableButton.setActionCommand(CMD_USER_TABLE);
        place(gbc, pane, userTableButton,
              0, 1, 1, 1, 0.0, 0.0, gbc.CENTER, gbc.HORIZONTAL);
        tableTypeGroup.add(userTableButton);
        systemTableButton = new JToggleButton("VXeEe[u");
        systemTableButton.setActionCommand(CMD_SYSTEM_TABLE);
        tableTypeGroup.add(systemTableButton);
        place(gbc, pane, systemTableButton,
              0, 2, 1, 1, 0.0, 0.0, gbc.CENTER, gbc.HORIZONTAL);
        JPanel tablePanel = new JPanel();
        tablePanel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
        tableDisplayPanel = new JPanel();
        tableDisplayPanel.setLayout(new GridLayout(-1, 5, 1, 1));
        tablePanel.add(tableDisplayPanel);
        displayScrollPane = new JScrollPane(tablePanel);
        place(gbc, pane, displayScrollPane,
              1, 0, 3, 4, 1.0, 1.0, gbc.CENTER, gbc.BOTH);
        // j[o[̒ǉ
        menu = new JMenuBar();
        connectMenu = new JMenu("ڑ");
        connectItem = new JMenuItem("ڑ...");
        connectItem.setActionCommand(CMD_CONNECT);
        cutOffItem = new JMenuItem("ؒf");
        cutOffItem.setActionCommand(CMD_CUTOFF);
        propertyItem = new JMenuItem("vpeB...");
        propertyItem.setActionCommand(CMD_PROPERTY);
        exitItem = new JMenuItem("I");
        exitItem.setActionCommand(CMD_EXIT);
        connectMenu.add(connectItem);
        connectMenu.add(cutOffItem);
        connectMenu.add(propertyItem);
        connectMenu.add(new JSeparator());
        connectMenu.add(exitItem);
        menu.add(connectMenu);
        setJMenuBar(menu);
        //
        userTableButton.addActionListener(this);
        systemTableButton.addActionListener(this);
        connectItem.addActionListener(this);
        cutOffItem.addActionListener(this);
        propertyItem.addActionListener(this);
        exitItem.addActionListener(this);
        //
        pack();
        setSize(600, 400);
        validate();
    }

    /**
     * ActionEvent:action-performed B
     * @param ev Cxg
     */
    public void actionPerformed(ActionEvent ev) {
        String cmd = ev.getActionCommand();
        if (cmd.equals(CMD_USER_TABLE)) {
            application.setupType(DBBrowser.TYPE_USER_TABLE);
        } else if (cmd.equals(CMD_SYSTEM_TABLE)) {
            application.setupType(DBBrowser.TYPE_SYSTEM_TABLE);
        } else if (cmd.equals(CMD_CONNECT)) {
            application.getDBConnection();
        } else if (cmd.equals(CMD_CUTOFF)) {
            application.dropDBConnection();
        } else if (cmd.equals(CMD_PROPERTY)) {
            application.setDBProperty();
        } else if (cmd.equals(CMD_EXIT)) {
            application.stop();
        } else if (tableButtons.contains(ev.getSource())) {
            application.showTable(cmd);
        }
    }

    /* -- application method -- */
    /**
     * e[uꗗ\B
     * @param tables e[uꗗ
     */
    protected void displayTables(String[] tables) {
        tableDisplayPanel.removeAll();
        tableButtons.removeAll(tableButtons);
        displayScrollPane.revalidate();
        if (tables == null || tables.length < 1) {
            return;
        }
        //
        ImageIcon tableImage;
        JButton tableButton;
        tableImage = new ImageIcon("table.gif");
        if (tableImage.getIconHeight() < 1 ||
            tableImage.getIconWidth() < 1) {
            tableImage = null;
            for (int i = 0; i < tables.length;  i++) {
                tableButton = new JButton(tables[i]);
                tableButton.setBorderPainted(false);
                tableButton.setActionCommand(tables[i]);
                tableButton.addActionListener(this);
                tableButtons.add(tableButton);
                tableDisplayPanel.add(tableButton);
            }
        } else {
            for (int i = 0; i < tables.length; i++) {
                tableButton = new JButton(tables[i], tableImage);
                tableButton.setHorizontalTextPosition(JButton.CENTER);
                tableButton.setVerticalTextPosition(JButton.BOTTOM);
                tableButton.setBorderPainted(false);
                tableButton.setActionCommand(tables[i]);
                tableButton.addActionListener(this);
                tableButtons.add(tableButton);
                tableDisplayPanel.add(tableButton);
            }
        }
        displayScrollPane.revalidate();
    }
            
            

    /* -- utility method -- */
    /**
     * GUI Ŝ̎gp/sݒ肷B
     * @param f tO; true ȂgpB
     */
    protected void setEnabledAll(boolean f) {
        setEnabledAll(getContentPane(), f);
    }

    /**
     * w肳ꂽ GUI Rei̎gp/sݒ肷B
     * @param cn Rei
     * @param f tO
     */
    private void setEnabledAll(Container cn, boolean f) {
        Component[] coms = cn.getComponents();
        for(int i = 0; i < coms.length; i++){
            coms[i].setEnabled(f);
            if (coms[i] instanceof java.awt.Container) {
                setEnabledAll((Container)coms[i], f);
            }
        }
    }

    /**
     * G[_CAOŕ\B
     * @param mes bZ[W
     */
    protected void showError(String mes) {
        JOptionPane.showMessageDialog(this, mes, "ERROR",
                                      JOptionPane.ERROR_MESSAGE);
    }

    /**
     * GirdBagLayout zu\bhB
     */
    private void place(GridBagConstraints c, Container cn,
                       Component cmp, int x, int y, int w, int h,
                       double wx, double wy, int ac, int fl) {
        c.gridx = x;
        c.gridy = y;
        c.gridwidth = w;
        c.gridheight = h;
        c.weightx = wx;
        c.weighty = wy;
        c.anchor = ac;
        c.fill = fl;
        cn.add(cmp, c);
    }
}
