/*
 * Copyright (c) 1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, and distribute this software for
 * NON-COMMERCIAL purposes and without fee is hereby granted
 * provided that this copyright notice appears in all copies.
 *
 */

import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;

/**
 * vpeBҏW UIB
 * @version     1.1, 99/09/24
 * @author      Tadayoshi Maekawa, Sun Microsystems K.K.
 */
class PropertyEditorUI extends JDialog
implements ActionListener, ListSelectionListener {
    /**
     * ꗗe[ũwb_B
     */
    private static final String[] HEADER = {
        "vpeB", "l"
    };

    /**
     * GUIB
     */
    JTable listTable;
    JScrollPane listScrollPane;
    JTextField nameField;
    JTextField valueField;

    JButton registerButton;
    JButton removeButton;

    JMenuBar menu;
    JMenu operationMenu;
    JMenuItem closeItem;

    /**
     * vpeBҏWAvP[V{́B
     */
    PropertyEditor application;

    /**
     * ꗗ̃CfbNXpxN^B
     */
    Vector names;

    /**
     * action-commandB
     */
    private final String CMD_CLOSE = "CLOSE";
    private final String CMD_REGISTER = "REGISTER";
    private final String CMD_REMOVE = "REMOVE";

    /**
     * et[ƃ[_̃tOAAvP[V{̂
     * RXgN^B
     * @param pr et[
     * @param mf [_̃tO
     * @param ap vpeBҏWAvP[V{
     */
    PropertyEditorUI(Frame pr, boolean mf, PropertyEditor ap) {
        super (pr, "Property Editor", mf);
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        application = ap;
        names = new Vector();
        //
        GridBagLayout gbl = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();
        Container pane = getContentPane();
        pane.setLayout(gbl);
        gbc.insets = new Insets(5, 5, 5, 5);
        //
        listTable = new JTable(5, 2);
        listTable.setModel(new DefaultTableModel() {
            public boolean isCellEditable(int x, int y) {
                return false;
            }
        });
        listTable.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
        setPropertiesListTable(null);
        listScrollPane = new JScrollPane(listTable);
        place(gbc, pane, listScrollPane,
              0, 0, 4, 4, 1.0, 1.0, gbc.CENTER, gbc.BOTH);
        gbc.insets = new Insets(3, 5, 3, 5);
        nameField = new JTextField(20);
        place(gbc, pane, new JLabel("O", JLabel.LEFT),
              0, 4, 1, 1, 0.0, 0.0, gbc.WEST, gbc.NONE);
        place(gbc, pane, nameField,
              0, 5, 2, 1, 1.0, 0.0, gbc.WEST, gbc.NONE);
        valueField = new JTextField(20);
        place(gbc, pane, new JLabel("l", JLabel.LEFT),
              2, 4, 1, 1, 0.0, 0.0, gbc.WEST, gbc.NONE);
        place(gbc, pane, valueField,
              2, 5, 2, 1, 1.0, 0.0, gbc.WEST, gbc.NONE);
        registerButton = new JButton("o^");
        removeButton = new JButton("폜");
        registerButton.setActionCommand(CMD_REGISTER);
        removeButton.setActionCommand(CMD_REMOVE);
        place(gbc, pane, registerButton,
              0, 6, 2, 1, 0.0, 0.0, gbc.EAST, gbc.NONE);
        place(gbc, pane, removeButton,
              2, 6, 2, 1, 0.0, 0.0, gbc.WEST, gbc.NONE);
        // j[o[̒ǉ
        menu = new JMenuBar();
        operationMenu = new JMenu("");
        closeItem = new JMenuItem("");
        closeItem.setActionCommand(CMD_CLOSE);
        operationMenu.add(closeItem);
        menu.add(operationMenu);
        setJMenuBar(menu);
        //
        closeItem.addActionListener(this);
        registerButton.addActionListener(this);
        removeButton.addActionListener(this);
        listTable.getSelectionModel().addListSelectionListener(this);
        //
        pack();
    }

    /**
     * ActionEvent:action-performed B
     * @param ev Cxg
     */
    public void actionPerformed(ActionEvent ev) {
        String cmd = ev.getActionCommand();
        if (cmd.equals(CMD_CLOSE)) {
            dispose();
        } else if (cmd.equals(CMD_REGISTER)) {
            addProperty(nameField.getText().trim(),
                        valueField.getText().trim());
        } else if (cmd.equals(CMD_REMOVE)) {
            removeProperty(nameField.getText().trim());
        }
    }

    /**
     * ListSelectionEvent:value-changed B
     * @param ev Cxg
     */
    public void valueChanged(ListSelectionEvent ev) {
        if (ev.getValueIsAdjusting()) {
            return;
        }
        displaySelectedProperty();
    }

    /* -- application method -- */
    /**
     * vpeBꗗe[uɐݒ肷B
     * @param pr vpeB
     */
    protected void setPropertiesListTable(Properties pr) {
        DefaultTableModel tm = (DefaultTableModel)listTable.getModel();
        names.removeAll(names);
        tm.setDataVector(new Object[0][0], HEADER);
        listTable.sizeColumnsToFit(-1);
        if (pr == null || pr.size() < 1) {
            return;
        }
        Iterator nms;
        Object[][] rs = new Object[pr.size()][2];
        int i;
        for (i = 0, nms = pr.keySet().iterator(); nms.hasNext(); i++) {
            rs[i][0] = nms.next();
            rs[i][1] = pr.get(rs[i][0]);
            names.add(rs[i][0]);
        }
        tm.setDataVector(rs, HEADER);
    }

    /**
     * ꗗe[uvpeB擾B
     */
    protected Properties getPropertiesListTable() {
        DefaultTableModel tm = (DefaultTableModel)listTable.getModel();
        Properties ret = new Properties();
        for (int i = 0; i < tm.getRowCount(); i++) {
            ret.put(tm.getValueAt(i, 0), tm.getValueAt(i, 1));
        }
        return ret;
    }

    /**
     * vpeBꗗe[uɓo^B
     * @param n O
     * @param v l
     */
    private void addProperty(String n, String v) {
        DefaultTableModel tm = (DefaultTableModel)listTable.getModel();
        int ix = names.indexOf(n);
        if (ix >= 0) {
            tm.setValueAt(v, ix, 1);
        } else {
            tm.addRow(new Object[] { n, v });
            names.add(n);
            ix = tm.getRowCount() -1;
        }
        listTable.setRowSelectionInterval(ix, ix);
        listTable.scrollRectToVisible(
            ((Component)listTable.getCellRenderer(ix, 0)).getBounds());
    }

    /**
     * vpeBꗗe[u폜B
     * @param n O
     */
    private void removeProperty(String n) {
        DefaultTableModel tm = (DefaultTableModel)listTable.getModel();
        int ix = names.indexOf(n);
        if (ix < 0) {
            showError("O '"+ n +"' ̃vpeB͂܂B");
            return;
        }
        names.remove(ix);
        tm.removeRow(ix);
    }

    /**
     * ꗗe[uőIĂvpeBtB[hɕ\B
     */
    private void displaySelectedProperty() {
        int ix = listTable.getSelectedRow();
        if (ix >= 0) {
            String n, v;
            DefaultTableModel tm = (DefaultTableModel)listTable.getModel();
            n = (String)tm.getValueAt(ix, 0);
            v = (String)tm.getValueAt(ix, 1);
            nameField.setText(n);
            valueField.setText(v);
        }
    }

    /* -- utility method -- */
    /**
     * GirdBagLayout zu\bhB
     */
    private void place(GridBagConstraints c, Container cn,
                       Component cmp, int x, int y, int w, int h,
                       double wx, double wy, int ac, int fl) {
        c.gridx = x;
        c.gridy = y;
        c.gridwidth = w;
        c.gridheight = h;
        c.weightx = wx;
        c.weighty = wy;
        c.anchor = ac;
        c.fill = fl;
        cn.add(cmp, c);
    }

    /**
     * G[_CAOŕ\B
     * @param mes bZ[W
     */
    protected void showError(String mes) {
        JOptionPane.showMessageDialog(this, mes, "ERROR",
                                      JOptionPane.ERROR_MESSAGE);
    }
}
