/*
 * Copyright (c) 1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, and distribute this software for
 * NON-COMMERCIAL purposes and without fee is hereby granted
 * provided that this copyright notice appears in all copies.
 *
 */

import java.sql.*;
import java.util.Vector;
import java.util.Hashtable;

/**
 * ėpe[uҏWAvP[VB
 * @author      Tadayoshi Maekawa, Sun Microsystems K.K.
 * @version     1.2 99/09/22
 */
class TableViewer {
    /**
     * 萔:
     */

    /**
     * eAvP[Vւ̃t@XB
     */
    DBBrowser parent;

    /**
     * \Ώۂ̃e[uB
     */
    String tableName;

    /**
     * GUI ւ̃t@XB
     */
    TableViewerUI ui;

    /**
     * f[^x[XڑB
     */
    Connection conn;

    /**
     * JꗗB
     */
    Vector columns;

    /**
     * Eꗗ
     */
    Vector rows;

    /**
     * RXgN^B
     * @param pt eAvP[V(DBBrowser)
     * @param cn DB ڑ
     * @param tn e[u
     */
    protected TableViewer(DBBrowser pt, Connection cn, String tn) {
        parent = pt;
        tableName = tn;
        conn = cn;
        columns = new Vector();
        rows = new Vector();
        // GUI
        ui = new TableViewerUI(this, tn);
        ui.setVisible(true);
    }

    /**
     * NBAvP[ṼtbV(restart)ɂgpB
     */
    protected void start() {
        // e[u & \
        try {
            reloadList();
        } catch (CanNotReloadTableRowsException ex) {
            ex.printStackTrace();
            ui.showError("e[ǔɎs܂B\n"+
                         "e[uEGfB^I܂B");
            stop();
            return;
        }
    }

    /**
     * IB
     */
    protected void stop() {
        if (ui != null) {
            try {
                ui.dispose();
            } catch (Exception ex) {}
            ui = null;
        }
        parent.hidedTable(tableName);
    }

    /* -- application method -- */
    /**
     * e[ũEꗗ擾B
     * Eꗗɂ CHAR n INTEGER ñf[^^̃JB
     * @exception CanNotReloadTableRowsException Eꗗ̌Ɏs
     * ꍇB
     */
    private synchronized void reloadList()
    throws CanNotReloadTableRowsException {
        columns.removeAll(columns);
        rows.removeAll(rows);
        try {
            ResultSet rs;
            ResultSetMetaData rsmd;
            Statement stmt;
            TableViewerColumn cl;
            Vector rw;
            int i;
            //
            stmt = conn.createStatement();
            rs = stmt.executeQuery("SELECT * FROM " + tableName);
            rsmd = rs.getMetaData();
            for (i = 1; i <= rsmd.getColumnCount(); i++) {
                String lb, nm;
                int tp, ds, nl;
                lb = rsmd.getColumnLabel(i);
                nm = rsmd.getColumnName(i);
                tp = rsmd.getColumnType(i);
                ds = rsmd.getColumnDisplaySize(i);
                nl = rsmd.isNullable(i);
                cl = new TableViewerColumn(lb, nm, tp, ds,
                                           (nl == rsmd.columnNullable));
                columns.add(cl);
            }
            while (rs.next()) {
                final String NULL = "null";
                Object to;
                rw = new Vector();
                for (i = 1; i <= columns.size(); i++) {
                    cl = (TableViewerColumn)columns.get(i-1);
                    switch (cl.type) {
                    case Types.BIT:
                        to = String.valueOf(""+ rs.getBoolean(i));
                        break;
                    case Types.BINARY:
                    case Types.VARBINARY:
                        rs.getBytes(i);
                        to = "[(VAR)BINARY]";
                        break;
                    case Types.BIGINT:
                    case Types.DECIMAL:
                    case Types.NUMERIC:
                        to = rs.getBigDecimal(i);
                        break;
                    case Types.INTEGER:
                    case Types.SMALLINT:
                        to = new Integer(rs.getInt(i));
                        break;
                    case Types.CHAR:
                    case Types.VARCHAR:
                        to = rs.getString(i);
                        break;
                    case Types.DATE:
                        to = rs.getDate(i);
                        break;
                    case Types.DOUBLE:
                    case Types.FLOAT:
                        to = new Double(rs.getDouble(i));
                        break;
                    case Types.LONGVARBINARY:
                        //rs.getBlob(i).getBinaryStream();
                        to = "[LONGVARBINARY]";
                        break;
                    case Types.LONGVARCHAR:
                        //rs.getClob(i).getCharacterStream();
                        to = "[LONGVARCHAR]";
                        break;
                    case Types.REAL:
                        to = new Float(rs.getFloat(i));
                        break;
                    case Types.TIME:
                        to = rs.getTime(i);
                        break;
                    case Types.TIMESTAMP:
                        to = rs.getTimestamp(i);
                        break;
                    case Types.TINYINT:
                        to = new Byte(rs.getByte(i));
                        break;
                    case Types.OTHER:
                    case Types.NULL:
                    default:
                        to = NULL;
                        break;
                    }
                    if (rs.wasNull()) {
                        to = NULL;
                    } else {
                        if (to instanceof String) {
                            to = ((String)to).trim();
                        }
                        cl.maxDisplaySize = 
                            Math.max(cl.maxDisplaySize,
                                     to.toString().length());
                    }
                    rw.add(to);
                }
                rows.add(rw);
            }
            rs.close();
        } catch (SQLException ex) {
            throw new CanNotReloadTableRowsException(
                "Eꗗ̎擾Ɏs܂B", ex);
        }
        ui.setTableData(rows, columns);
    }
}
