/*
 * Copyright (c) 1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, and distribute this software for
 * NON-COMMERCIAL purposes and without fee is hereby granted
 * provided that this copyright notice appears in all copies.
 *
 */

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

/**
 * e[uEGfB^ GUI 񋟂B
 * @author      Tadayoshi Maekawa, Sun Microsystems K.K.
 * @version     1.2 99/09/22
 */
class TableViewerUI extends JFrame implements ActionListener {
    /**
     * AvP[V{(e[uEGfB^)B
     */
    TableViewer application;

    /**
     * GUI IuWFNgB
     */
    JTable listTable;
    JScrollPane tableScrollPane;

    JMenuBar menu;
    JMenu operationMenu;
    JMenuItem reloadItem;
    JMenuItem closeItem;

    /**
     * action-commandB
     */
    final String CMD_RELOAD = "RELOAD";
    final String CMD_CLOSE = "CLOSE";

    /**
     * RXgN^B
     * @param ap AvP[V{
     * @param nm Ώۃe[u
     */
    public TableViewerUI(TableViewer ap, String nm) {
        super("TableViewer: " +nm);
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        application = ap;
        //
        GridBagLayout gbl = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();
        Container pane = getContentPane();
        pane.setLayout(gbl);
        //
        gbc.insets = new Insets(5, 5, 5, 5);
        listTable = new JTable();
        listTable.setModel(new DefaultTableModel() {
            public boolean isCellEditable(int x, int y) {
                return false;
            }
        });
        listTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        tableScrollPane = new JScrollPane(listTable);
        place(gbc, pane, tableScrollPane,
              0, 0, 1, 1, 1.0, 1.0, gbc.CENTER, gbc.BOTH);
        // j[o[̒ǉ
        menu = new JMenuBar();
        operationMenu = new JMenu("");
        reloadItem = new JMenuItem("ēǍ");
        reloadItem.setActionCommand(CMD_RELOAD);
        closeItem = new JMenuItem("");
        closeItem.setActionCommand(CMD_CLOSE);
        operationMenu.add(reloadItem);
        operationMenu.add(new JSeparator());
        operationMenu.add(closeItem);
        menu.add(operationMenu);
        setJMenuBar(menu);
        //
        reloadItem.addActionListener(this);
        closeItem.addActionListener(this);
        //
        pack();
        validate();
    }

    /**
     * ActionEvent:action-performed B
     * @param ev Cxg
     */
    public void actionPerformed(ActionEvent ev) {
        String cmd = ev.getActionCommand();
        if (cmd.equals(CMD_RELOAD)) {
            application.start();
        } else if (cmd.equals(CMD_CLOSE)) {
            application.stop();
        }
    }

    /* -- application method -- */
    /**
     * ꗗe[uɃf[^ݒ肷B
     * @param dt f[^Q
     * @param cs JQ
     */
    protected void setTableData(Vector dt, Vector cs) {
        TableViewerColumn tvc;
        DefaultTableModel tm = (DefaultTableModel)listTable.getModel();
        TableColumnModel cm = listTable.getColumnModel();
        TableColumn cl;
        Font f;
        int i, total, w, uw;
        //
        listTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        // PʕvZ
        f = listTable.getFont();
        if (f == null) {
            f = getFont();
        }
        uw = (f != null) ? getFontMetrics(f).charWidth('m') : 14;
        //
        tm.setDataVector(dt, cs);
        for (i = 0, total = 0; i < cs.size(); i++) {
            cl = cm.getColumn(i);
            tvc = (TableViewerColumn)cs.get(i);
            w = tvc.maxDisplaySize * uw;
            cl.setPreferredWidth(w);
            total += w;
        }
        if (tableScrollPane.getViewport().getSize().width > total) {
            listTable.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
            listTable.sizeColumnsToFit(-1);
        }
    }

    /* -- utility method -- */
    /**
     * G[_CAOŕ\B
     * @param mes bZ[W
     */
    protected void showError(String mes) {
        JOptionPane.showMessageDialog(this, mes, "ERROR",
                                      JOptionPane.ERROR_MESSAGE);
    }

    /**
     * GirdBagLayout zu\bhB
     */
    private void place(GridBagConstraints c, Container cn,
                       Component cmp, int x, int y, int w, int h,
                       double wx, double wy, int ac, int fl) {
        c.gridx = x;
        c.gridy = y;
        c.gridwidth = w;
        c.gridheight = h;
        c.weightx = wx;
        c.weighty = wy;
        c.anchor = ac;
        c.fill = fl;
        cn.add(cmp, c);
    }
}
