/*
 * Copyright (c) 1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, and distribute this software for
 * NON-COMMERCIAL purposes and without fee is hereby granted
 * provided that this copyright notice appears in all copies.
 *
 */

import java.sql.*;
import java.util.*;
import javax.swing.ListModel;
import javax.swing.DefaultListModel;

/**
 * JDBCTerminal CEt[EW[
 * @version     1.3 99/10/13
 * @author      Yuichi Omata, Sun Microsystems Inc.
 * @author      Tadayoshi Maekawa, Sun Microsystems K.K.
 */
public class MainFrame {
    JDBCTerminal app;
    MainFrameGUI gui;
    MainFrameCommand command;
    LoginDialog loginDialog;

    Connection conn;

    /**
     *
     */
    public MainFrame(JDBCTerminal app){
        this.app = app;
        gui = new MainFrameGUI(this);
        updateDriverList();
        loginDialog = new LoginDialog(gui);
    }

    /**
     * start()
     */
    public void start() {
        gui.setVisible(true);
    }

    /**
     *
     * @return
     */
    public JDBCTerminal getApp(){
        return app;
    }

    /**
     * hCoEXgXVB
     */
    public void updateDriverList(){
        gui.driverList.removeAll();

        // hCoE}l[Wɓo^ĂhCo GUI ɕ\
        DefaultListModel lm = (DefaultListModel)gui.driverList.getModel();
        for (Enumeration e = DriverManager.getDrivers();e.hasMoreElements();){
            lm.addElement(e.nextElement().getClass().getName());
        }
    }

    /**
     * w肳ꂽhCohCo̎̂擾B
     * @return hCo
     */
    private Driver getDriverFromName(String drvName){
        Driver drv = null;

        for (Enumeration e = DriverManager.getDrivers();e.hasMoreElements();){
            drv = (Driver)e.nextElement();
            if (drvName.equals(drv.getClass().getName()))
                break;
        }

        return drv;
    }

    /**
     * hCỏ
     */
    public void deregisterDriver(){

        // XgIĂhCo擾
        Driver drv;
        drv = getDriverFromName((String)gui.driverList.getSelectedValue());

        if (drv == null) return;

        try {
            // hCỏƃXg̍XV
            DriverManager.deregisterDriver(drv);
            updateDriverList();
        } catch (SQLException ex) {
            JDBCTerminal.showMessage(
                "Could not deregister driver " + drv.toString() +"\n"+
                "Message: " + ex.getMessage() +"\n"+
                "State: " + ex.getSQLState() +"\n"+
                "Error code: "  + ex.getErrorCode(), gui);
        }
    }

    /**
     *
     */
    public void Quit() {
        gui.setVisible(false);
        gui.dispose();
        System.exit(0);
    }

    /**
     *
     */
    public void showStatementWindow() {
        app.getStatementFrame().setVisible(true);
    }

    /**
     *
     */
    public void showLogWindow() {
        app.getLogFrame().setVisible(true);
        DriverManager.setLogWriter(app.getLogFrame().getLogWriter());
    }

    /**
     *
     */
    public void showAddDriverDialog() {
        getApp().getAddDriverDialog().setVisible(true);
    }

    /**
     * f[^x[Xւ̐ڑ̊msB
     */
    public void Connect(){
        loginDialog.setVisible(true);
        Properties prop = loginDialog.getProp();

        Driver drv = null;

        try {
            if (gui.driverChoiceTypeCheckboxGroup.getSelection().equals(
                    gui.driverManagerCheckbox.getModel())){
                // hCȏIhCoE}l[WɔCꍇ
                conn = DriverManager.getConnection(
                    gui.URLTextField.getText(), prop);
            } else {
                // hCow肷ꍇ
                // IꂽhCo擾
                drv = getDriverFromName(
                    (String)gui.driverList.getSelectedValue());
                if (drv == null) return;
                // hCoɒڎwĐڑs
                conn = drv.connect(gui.URLTextField.getText(), prop);
            }
            getApp().getStatementFrame().setConnection(conn);
            getApp().getStatementFrame().setVisible(true);
        } catch (java.sql.SQLException ex) {
            JDBCTerminal.showMessage(
                "Cannot connect to DBMS.\n" +
                "Message: " + ex.getMessage() +"\n"+
                "State: " + ex.getSQLState() +"\n"+
                "Error code: "  + ex.getErrorCode(), gui);
        }
    }

}
