/*
 * Copyright (c) 1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, and distribute this software for
 * NON-COMMERCIAL purposes and without fee is hereby granted
 * provided that this copyright notice appears in all copies.
 *
 */

import java.awt.*;
import javax.swing.*;

/**
 * JDBCTerminal CEt[ GUI
 * @version	1.2 99/10/12
 * @author 	Yuichi Omata, Sun Microsystems Inc.
 * @author      Tadayoshi Maekawa, Sun Microsystems K.K.
 */
public class MainFrameGUI extends JFrame {

    MainFrame mainFrame;
    JLabel URLLabel;
    JTextField URLTextField;
    JButton connectButton;
    JLabel driverLabel;
    JList driverList;
    JLabel driverChoiceTypeLabel;

    JPanel driverChoiceTypePanel;
    ButtonGroup driverChoiceTypeCheckboxGroup;
    JCheckBox selectedDriverCheckbox;
    JCheckBox driverManagerCheckbox;

    JButton registerDriverButton;
    JButton deregisterDriverButton;

    JMenuBar MainMenuBar;
    JMenu FileMenu;
    JMenu WindowMenu;
    JMenuItem FileMenuQuit;
    JMenuItem WindowMenuStatement;
    JMenuItem WindowMenuLog;

    public MainFrameGUI(MainFrame mainFrame) {
	this.mainFrame = mainFrame;
	setTitle("JDBCTerminal");
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);

        Container pane = getContentPane();
	GridBagLayout gbl = new GridBagLayout();
	gbl.columnWidths = new int[] {14,14,14,14,14,14,14};
	gbl.rowHeights =  new int[] {14,14,14,14,14,14,14,14};
	gbl.columnWeights = new double[] {0.0,0.0,0.0,0.0,0.0,0.0,0.0};
	gbl.rowWeights = new double[] {0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0};
	pane.setLayout(gbl);
	GridBagConstraints gbc = new GridBagConstraints();

	// 
	URLLabel = new JLabel("JDBC URL:");
	gbc.gridx = 1;
	gbc.gridy = 1;
	gbc.anchor = GridBagConstraints.WEST;
	gbl.setConstraints(URLLabel, gbc);
	pane.add(URLLabel);

	// Create URL TextField
	URLTextField = new JTextField(35);
	gbc.gridy = 2;
	gbl.setConstraints(URLTextField, gbc);
	pane.add(URLTextField);

	// Create driverLabel
	driverLabel = new JLabel("Loaded Drivers:");
	gbc.gridy = 4;
	gbl.setConstraints(driverLabel, gbc);
	pane.add(driverLabel);

	// Create driver list
        JScrollPane driverPane;
	driverList = new JList(new DefaultListModel());
        driverList.setVisibleRowCount(5);
        driverPane = new JScrollPane(driverList);
	gbc.gridy = 5;
	gbc.fill = GridBagConstraints.BOTH;
	gbl.setConstraints(driverPane, gbc);
	pane.add(driverPane);
	
	// create Connect Button
	connectButton = new JButton("Connect...");
	gbc.gridx = 3;
	gbc.gridy = 2;
	gbc.gridwidth = 3;
	gbc.fill = GridBagConstraints.NONE;
	gbl.setConstraints(connectButton, gbc);
	connectButton.addActionListener(new MainFrameCommand(MainFrameCommand.Connect, mainFrame));
	pane.add(connectButton);

	//
	driverChoiceTypeLabel = new JLabel("Driver Selection:");
        gbc.gridy = 4;
	gbl.setConstraints(driverChoiceTypeLabel, gbc);
	pane.add(driverChoiceTypeLabel);

	// driverChoicePanel
	driverChoiceTypePanel = new JPanel();
	GridBagLayout dctpGbl = new GridBagLayout();
	dctpGbl.columnWidths = new int[] {14};
	dctpGbl.rowHeights =  new int[] {14,14};
	dctpGbl.columnWeights = new double[] {0.0};
	dctpGbl.rowWeights = new double[] {0.0,0.0,0.0,0.0};
	driverChoiceTypePanel.setLayout(dctpGbl);

	GridBagConstraints dctpGbc = new GridBagConstraints();

	driverChoiceTypeCheckboxGroup = new ButtonGroup();
	driverManagerCheckbox = new JCheckBox("Use Driver Manager", true);
        driverChoiceTypeCheckboxGroup.add(driverManagerCheckbox);
	dctpGbc.anchor = GridBagConstraints.WEST;
	dctpGbl.setConstraints(driverManagerCheckbox, dctpGbc);
	driverChoiceTypePanel.add(driverManagerCheckbox);

	selectedDriverCheckbox = new JCheckBox("Selected Driver", false);
        driverChoiceTypeCheckboxGroup.add(selectedDriverCheckbox);
	dctpGbc.gridy = 1;
	dctpGbl.setConstraints(selectedDriverCheckbox, dctpGbc);
	driverChoiceTypePanel.add(selectedDriverCheckbox);

	gbc.gridy = 5;
	gbc.anchor = GridBagConstraints.NORTHWEST;
	gbl.setConstraints(driverChoiceTypePanel, gbc);
	pane.add(driverChoiceTypePanel);

	// add Two Buttons
	registerDriverButton = new JButton("Add Driver...");
	gbc.gridy = 6;
	gbc.gridwidth = 1;
	gbc.anchor = GridBagConstraints.WEST;
	gbl.setConstraints(registerDriverButton, gbc);
	registerDriverButton.addActionListener(new MainFrameCommand(MainFrameCommand.RegisterDriver, mainFrame));
	pane.add(registerDriverButton);

	deregisterDriverButton = new JButton("Delete Driver");
	gbc.gridx = 5;
	gbl.setConstraints(deregisterDriverButton, gbc);
	deregisterDriverButton.addActionListener(new MainFrameCommand(MainFrameCommand.DeregisterDriver, mainFrame));
	pane.add(deregisterDriverButton);

	// Add Menus
	MainMenuBar = new JMenuBar();
	FileMenu = new JMenu("File");
	FileMenuQuit = new JMenuItem("Quit");
	FileMenuQuit.addActionListener(new MainFrameCommand(MainFrameCommand.FileMenuQuit, mainFrame));
	FileMenu.add(FileMenuQuit);
	MainMenuBar.add(FileMenu);

	WindowMenu = new JMenu("Window");
	WindowMenuStatement = new JMenuItem("SQL Statement");
	WindowMenuStatement.addActionListener(new MainFrameCommand(MainFrameCommand.WindowMenuStatement, mainFrame));
	WindowMenu.add(WindowMenuStatement);
	WindowMenuLog = new JMenuItem("Error/Warning");
	WindowMenuLog.addActionListener(new MainFrameCommand(MainFrameCommand.WindowMenuLog, mainFrame));
	WindowMenu.add(WindowMenuLog);
	MainMenuBar.add(WindowMenu);

	setJMenuBar(MainMenuBar);

	pack();
    }
}
